class SpecConfig:
    META = '.meta'
    TABLE = 'table-name'

    TYPE = 'type'
    COLUMN = 'column'

    ID = 'id'
    SEQUENCE = 'sequence'

    RELATION = 'relation'
    JOIN_TABLE = 'join-table'
    JOIN_COLUMN = 'join-column'
    INVERSE_JOIN_COLUMN = 'inverse-join-column'
    MAPPED_BY = 'mapped-by'
