@OneToMany(mappedBy = "{{ mapped_by }}", cascade = {CascadeType.PERSIST, CascadeType.MERGE}{% if remove_orphan -%} , orphanRemoval = true {% endif %})
@Builder.Default
private {{ collection_type }}<{{ class_type }}> {{ name }} = new {{ collection_type_impl }}<>();
